from contextlib import contextmanager
from io import BytesIO
from pathlib import Path
from typing import Any
from typing import Generator
from typing import Generic
from typing import IO
from typing import Iterator
from typing import Type
from typing import TypeVar

import orjson
from pydantic import BaseModel
from pydantic import Field


def orjson_dumps(value: Any, *, default: Any) -> str:
    return orjson.dumps(value, default=default).decode()


class TelegramBotApiType(BaseModel):
    class Config:
        allow_population_by_field_name = True
        extra = "forbid"
        json_dumps = orjson_dumps
        json_loads = orjson.loads

    def _prepare_export_kw(self, kw: dict[str, Any]) -> None:
        kw.update(
            {
                "by_alias": True,
                "exclude_none": True,
                "exclude_unset": True,
            }
        )

    def jsonb(self, **kw: Any) -> bytes:  # noqa: A003, VNE003
        value = self.json(**kw)
        if isinstance(value, bytes):
            return value

        return value.encode()

    def json(self, **kw: Any) -> str:  # noqa: A003, VNE003
        self._prepare_export_kw(kw)

        value: str | bytes = super().json(**kw)
        if isinstance(value, str):
            return value

        return value.decode()

    def dict(self, **kw: Any) -> dict:  # noqa: A003, VNE003
        self._prepare_export_kw(kw)
        return super().dict(**kw)


class Request(TelegramBotApiType):
    @contextmanager
    def files(self) -> Iterator[dict[str, IO]]:
        opened_files: list[IO] = []

        def open_file(_path_or_io: Path | IO) -> IO:
            if isinstance(_path_or_io, BytesIO):
                return _path_or_io
            assert isinstance(_path_or_io, Path)
            _fp = _path_or_io.open("rb")
            opened_files.append(_fp)
            return _fp

        try:
            fields_streams = {
                field: open_file(value)
                for field, value in self._get_input_files().items()
            }

            yield fields_streams
        finally:
            for fp in opened_files:
                fp.close()

    def _get_input_files(self) -> dict[str, Path | IO]:
        fields_values: Generator[tuple[str, Any], None, None] = (
            (attr, getattr(self, attr, None)) for attr in self.__fields__
        )

        fields_files: Generator[tuple[str, Path | IO], None, None] = (
            field_value
            for field_value in fields_values
            if isinstance(field_value[1], (Path, BytesIO))
        )

        return dict(fields_files)

    def _prepare_export_kw(self, kw: dict[str, Any]) -> None:
        kw["exclude"] = frozenset(self._get_input_files())
        return super()._prepare_export_kw(kw)


class ResponseParameters(TelegramBotApiType):
    migrate_to_chat_id: None | int = Field(None)
    retry_after: None | int = Field(None)


ResponseResultT = TypeVar("ResponseResultT")


class Response(Generic[ResponseResultT], TelegramBotApiType):
    """
    The response contains a JSON object,
        which always has a Boolean field 'ok'
        and may have an optional String field 'description'
        with a human-readable description of the result.
    If 'ok' equals true, the request was successful
        and the result of the query can be found
        in the 'result' field.
    In case of an unsuccessful request,
        'ok' equals false and the error is explained
        in the 'description'.
    An Integer 'error_code' field is also returned,
        but its contents are subject to change in the future.
    Some errors may also have an optional field 'parameters'
        of the type ResponseParameters,
        which can help to automatically handle the error.
    https://core.telegram.org/bots/api#making-requests
    """

    ok: bool = Field(...)
    result: None | ResponseResultT = Field(None)
    error_code: None | int = Field(None)
    description: None | str = Field(None)
    parameters: None | ResponseParameters = Field(None)


BaseModelType = Type[BaseModel]

__models__: set[Type[TelegramBotApiType]] = {
    Request,
    Response,
    ResponseParameters,
    TelegramBotApiType,
}

__all__ = (
    "__models__",
    "Request",
    "Response",
    "ResponseParameters",
    "TelegramBotApiType",
)



from typing import Any
from typing import Type

from pydantic import Field

from oyabun.telegram.base import Request
from oyabun.telegram.entities import InlineKeyboardMarkup
from oyabun.telegram.entities import MessageEntity
from oyabun.telegram.entities import ReplyMarkupType


class AnswerCallbackQueryRequest(Request):
    cache_time: None | int = Field(None)
    callback_query_id: str = Field(...)
    show_alert: None | bool = Field(None)
    text: None | str = Field(None)
    url: None | str = Field(None)


class DeleteWebhookRequest(Request):
    drop_pending_updates: None | bool = Field(None)


class EditMessageCaptionRequest(Request):
    caption: None | str = Field(None)
    caption_entities: None | list[MessageEntity] = Field(None)
    chat_id: None | int | str = Field(None)
    inline_message_id: None | str = Field(None)
    message_id: None | int = Field(None)
    parse_mode: None | str = Field(None)
    reply_markup: None | InlineKeyboardMarkup = Field(None)


class EditMessageReplyMarkupRequest(Request):
    chat_id: None | int | str = Field(None)
    inline_message_id: None | str = Field(None)
    message_id: None | int = Field(None)
    reply_markup: None | InlineKeyboardMarkup = Field(None)


class EditMessageTextRequest(Request):
    chat_id: None | int | str = Field(None)
    disable_web_page_preview: None | bool = Field(None)
    entities: None | list[MessageEntity] = Field(None)
    inline_message_id: None | str = Field(None)
    message_id: None | int = Field(None)
    parse_mode: None | str = Field(None)
    reply_markup: None | InlineKeyboardMarkup = Field(None)
    text: str = Field(...)


class GetChatRequest(Request):
    chat_id: int | str = Field(...)


class GetFileRequest(Request):
    file_id: str = Field(...)


class GetMeRequest(Request):
    pass


class GetUpdatesRequest(Request):
    allowed_updates: None | list[str] = Field(None)
    limit: None | int = Field(None, le=100, ge=1)
    offset: None | int = Field(None)
    timeout: None | int = Field(None, ge=0)


class GetWebhookInfoRequest(Request):
    pass


class SendMessageRequest(Request):
    allow_sending_without_reply: None | bool = Field(None)
    chat_id: int | str = Field(...)
    disable_notification: None | bool = Field(None)
    disable_web_page_preview: None | bool = Field(None)
    entities: None | list[MessageEntity] = Field(None)
    parse_mode: None | str = Field(None)
    reply_markup: None | ReplyMarkupType = Field(None)
    reply_to_message_id: None | int = Field(None)
    text: str = Field(...)


class SendPhotoRequest(Request):
    allow_sending_without_reply: None | bool = Field(None)
    caption: None | str = Field(None)
    caption_entities: None | list[MessageEntity] = Field(None)
    chat_id: int | str = Field(...)
    disable_notification: None | bool = Field(None)
    parse_mode: None | str = Field(None)
    photo: Any = Field(...)
    reply_markup: None | ReplyMarkupType = Field(None)
    reply_to_message_id: None | int = Field(None)


class SetWebhookRequest(Request):
    allowed_updates: None | list[str] = Field(None)
    certificate: Any = Field(None)
    drop_pending_updates: None | bool = Field(None)
    ip_address: None | str = Field(None)
    max_connections: None | int = Field(None)
    url: str = Field(...)


__models__: set[Type[Request]] = {
    AnswerCallbackQueryRequest,
    DeleteWebhookRequest,
    EditMessageCaptionRequest,
    EditMessageReplyMarkupRequest,
    EditMessageTextRequest,
    GetChatRequest,
    GetFileRequest,
    GetMeRequest,
    GetUpdatesRequest,
    GetWebhookInfoRequest,
    SendMessageRequest,
    SendPhotoRequest,
    SetWebhookRequest,
}

__all__ = (
    "__models__",
    "AnswerCallbackQueryRequest",
    "DeleteWebhookRequest",
    "EditMessageCaptionRequest",
    "EditMessageReplyMarkupRequest",
    "EditMessageTextRequest",
    "GetChatRequest",
    "GetFileRequest",
    "GetMeRequest",
    "GetUpdatesRequest",
    "GetWebhookInfoRequest",
    "SendMessageRequest",
    "SendPhotoRequest",
    "SetWebhookRequest",
)



from contextlib import asynccontextmanager
from io import BytesIO
from pathlib import Path
from typing import AsyncGenerator
from typing import IO
from typing import Type
from typing import TypeVar

import aiohttp
import orjson

from oyabun.telegram import AnswerCallbackQueryRequest
from oyabun.telegram import AnswerCallbackQueryResponse
from oyabun.telegram import Chat
from oyabun.telegram import DeleteWebhookResponse
from oyabun.telegram import EditMessageCaptionRequest
from oyabun.telegram import EditMessageCaptionResponse
from oyabun.telegram import EditMessageReplyMarkupRequest
from oyabun.telegram import EditMessageReplyMarkupResponse
from oyabun.telegram import EditMessageTextRequest
from oyabun.telegram import EditMessageTextResponse
from oyabun.telegram import File
from oyabun.telegram import GetChatRequest
from oyabun.telegram import GetChatResponse
from oyabun.telegram import GetFileRequest
from oyabun.telegram import GetFileResponse
from oyabun.telegram import GetMeResponse
from oyabun.telegram import GetUpdatesRequest
from oyabun.telegram import GetUpdatesResponse
from oyabun.telegram import GetWebhookInfoResponse
from oyabun.telegram import InlineKeyboardMarkup
from oyabun.telegram import Message
from oyabun.telegram import MessageEntity
from oyabun.telegram import SendMessageRequest
from oyabun.telegram import SendMessageResponse
from oyabun.telegram import SendPhotoRequest
from oyabun.telegram import SendPhotoResponse
from oyabun.telegram import SetWebhookRequest
from oyabun.telegram import SetWebhookResponse
from oyabun.telegram import Update
from oyabun.telegram import User
from oyabun.telegram import WebhookInfo
from oyabun.telegram.base import Request
from oyabun.telegram.base import Response
from oyabun.telegram.entities import ReplyMarkupType


class Bot:
    """
    The class represents an entrypoint to communicate with Telegram Bot API.

    The methods are synchronized with those from official API doc.

    Usage: instantiate using bot token and go.

    If something is wrong, you'll get the `Bot.RequestError` exception raised.
    """

    TELEGRAM_BOT_API_URL = "https://api.telegram.org"

    class RequestError(RuntimeError):
        """
        A base exception for any internal error, including those
        caused by malformed requests and invalid data.
        """

        pass

    def __init__(
        self,
        token: str,
        *,
        session: aiohttp.ClientSession | None = None,
    ):
        """
        Sets up the new Bot instance.

        :param token: a bot token which BotFather gives to you.
        :param session: existing ClientSession or None (bot will use its own)
        """

        self.__session = session
        self.__token = token

    @asynccontextmanager
    async def client_session(
        self,
    ) -> AsyncGenerator[aiohttp.ClientSession, None]:
        if self.__session:
            yield self.__session
            return

        async with aiohttp.ClientSession() as session:
            prev = self.__session
            try:
                self.__session = session
                yield self.__session
            finally:
                self.__session = prev

    @property
    def api_url(self) -> str:
        """
        An API URL to make requests to.

        :return: the completed API URL as a string
        """

        return f"{self.TELEGRAM_BOT_API_URL}/bot{self.__token}"

    @property
    def file_url(self) -> str:
        """
        A URL to download files from.

        :return: the completed URL as a string
        """

        return f"{self.TELEGRAM_BOT_API_URL}/file/bot{self.__token}"

    async def answerCallbackQuery(
        self,
        *,
        callback_query_id: str,
        text: None | str = None,
        show_alert: None | bool = None,
        url: None | str = None,
        cache_time: None | int = None,
    ) -> bool:
        request = AnswerCallbackQueryRequest(
            cache_time=cache_time,
            callback_query_id=callback_query_id,
            show_alert=show_alert,
            text=text,
            url=url,
        )

        return await self._call_api(
            "answerCallbackQuery",
            request,
            response_cls=AnswerCallbackQueryResponse,
        )

    async def deleteWebhook(self) -> bool:
        return await self._call_api(
            "deleteWebhook",
            response_cls=DeleteWebhookResponse,
        )

    async def downloadFile(self, *, file: File) -> BytesIO:
        """
        Downloads the file's content into the BytesIO object.

        https://core.telegram.org/bots/api#getfile
        https://core.telegram.org/bots/api#file

        :param file: a File object

        :return: a BytesIO object with content of the file
        """

        if not file.file_path:
            raise self.RequestError(f"file {file} has no file_path set")

        return await self._download_file(file.file_path)

    async def editMessageCaption(
        self,
        *,
        caption: None | str = None,
        caption_entities: None | list[MessageEntity] | None = None,
        chat_id: None | int | str | None = None,
        inline_message_id: None | str | None = None,
        message_id: None | int | None = None,
        parse_mode: None | str | None = None,
        reply_markup: None | InlineKeyboardMarkup | None = None,
    ) -> bool | Message:
        """
        Use this method to edit captions of messages.

        On success, if the edited message is not an inline message,
        the edited Message is returned, otherwise True is returned.

        https://core.telegram.org/bots/api#editmessagecaption
        """

        request = EditMessageCaptionRequest(
            caption=caption,
            caption_entities=caption_entities,
            chat_id=chat_id,
            inline_message_id=inline_message_id,
            message_id=message_id,
            parse_mode=parse_mode,
            reply_markup=reply_markup,
        )

        return await self._call_api(
            "editMessageCaption",
            request,
            response_cls=EditMessageCaptionResponse,
        )

    async def editMessageReplyMarkup(
        self,
        *,
        chat_id: None | int | str | None = None,
        inline_message_id: None | str | None = None,
        message_id: None | int | None = None,
        reply_markup: None | InlineKeyboardMarkup | None = None,
    ) -> bool | Message:
        """
        Use this method to edit only the reply markup of messages.

        On success, if the edited message is not an inline message,
        the edited Message is returned, otherwise True is returned.

        https://core.telegram.org/bots/api#editmessagereplymarkup
        """

        request = EditMessageReplyMarkupRequest(
            chat_id=chat_id,
            inline_message_id=inline_message_id,
            message_id=message_id,
            reply_markup=reply_markup,
        )

        return await self._call_api(
            "editMessageReplyMarkup",
            request,
            response_cls=EditMessageReplyMarkupResponse,
        )

    async def editMessageText(
        self,
        *,
        chat_id: None | int | str | None = None,
        disable_web_page_preview: None | bool = None,
        entities: None | list[MessageEntity] = None,
        inline_message_id: None | str = None,
        message_id: None | int = None,
        parse_mode: None | str = None,
        reply_markup: None | InlineKeyboardMarkup = None,
        text: str,
    ) -> bool | Message:
        """
        Use this method to edit text and game messages.

        On success, if the edited message is not an inline message,
        the edited Message is returned, otherwise True is returned.

        https://core.telegram.org/bots/api#editmessagetext
        """

        request = EditMessageTextRequest(
            chat_id=chat_id,
            disable_web_page_preview=disable_web_page_preview,
            entities=entities,
            inline_message_id=inline_message_id,
            message_id=message_id,
            parse_mode=parse_mode,
            reply_markup=reply_markup,
            text=text,
        )

        return await self._call_api(
            "editMessageText",
            request,
            response_cls=EditMessageTextResponse,
        )

    async def getChat(self, *, chat_id: int | str) -> Chat:
        """
        Use this method to get up-to-date information about the chat
        (current name of the user for one-on-one conversations,
        current username of a user, group or channel, etc.).

        Returns a Chat object on success.

        https://core.telegram.org/bots/api#getchat
        """

        request = GetChatRequest(
            chat_id=chat_id,
        )

        return await self._call_api(
            "getChat",
            request,
            response_cls=GetChatResponse,
        )

    async def getFile(self, *, file_id: str) -> File:
        """
        Use this method to get basic info about a file
        and prepare it for downloading.

        For the moment, bots can download files of up to 20MB in size.

        The file can then be downloaded
        via the link https://api.telegram.org/file/bot<token>/<file_path>,
        where <file_path> is taken from the response.

        It is guaranteed that the link will be valid for at least 1 hour.

        When the link expires,
        a new one can be requested by calling getFile again.

        https://core.telegram.org/bots/api#getfile

        :return: on success, a File object
        """

        request = GetFileRequest(file_id=file_id)

        return await self._call_api(
            "getFile",
            request,
            response_cls=GetFileResponse,
        )

    async def getMe(self) -> User:
        """
        A simple method for testing your bot auth token.

        https://core.telegram.org/bots/api#getme

        :return: basic information about the bot in form of a User object
        """

        return await self._call_api(
            "getMe",
            response_cls=GetMeResponse,
        )

    async def getUpdates(
        self,
        *,
        offset: None | int = None,
        limit: None | int = None,
        timeout: None | int = None,
        allowed_updates: None | list[str] = None,
    ) -> list[Update]:
        request = GetUpdatesRequest(
            allowed_updates=allowed_updates,
            limit=limit,
            offset=offset,
            timeout=timeout,
        )

        return await self._call_api(
            "getUpdates",
            request,
            response_cls=GetUpdatesResponse,
            timeout=timeout,
        )

    async def getWebhookInfo(self) -> WebhookInfo:
        return await self._call_api(
            "getWebhookInfo",
            response_cls=GetWebhookInfoResponse,
        )

    async def sendMessage(
        self,
        *,
        chat_id: int | str,
        text: str,
        parse_mode: None | str = None,
        entities: None | list[MessageEntity] = None,
        disable_web_page_preview: None | bool = None,
        disable_notification: None | bool = None,
        reply_to_message_id: None | int = None,
        allow_sending_without_reply: None | bool = None,
        reply_markup: None | ReplyMarkupType = None,
    ) -> Message:
        """
        Use this method to send text messages.

        https://core.telegram.org/bots/api#sendmessage

        :param chat_id: Unique identifier for the target chat
        or username of the target channel
        (in the format @channelusername).

        :param text: Text of the message to be sent,
        1-4096 characters after entities parsing.

        :param parse_mode: Mode for parsing entities in the message text.
        See formatting options for more details.

        :param entities: A JSON-serialized list of special entities
        that appear in message text,
        which can be specified instead of parse_mode.

        :param disable_web_page_preview: Disables link previews
        for links in this message.

        :param disable_notification: Sends the message silently.
        Users will receive a notification with no sound.

        :param reply_to_message_id: If the message is a reply,
        ID of the original message.

        :param allow_sending_without_reply: Pass True,
        if the message should be sent
        even if the specified replied-to message is not found.

        :param reply_markup: Additional interface options.
        A JSON-serialized object for an inline keyboard,
        custom reply keyboard,
        instructions to remove reply keyboard
        or to force a reply from the user.

        :return: on success, the Message sent.
        """

        request = SendMessageRequest(
            allow_sending_without_reply=allow_sending_without_reply,
            chat_id=chat_id,
            disable_notification=disable_notification,
            disable_web_page_preview=disable_web_page_preview,
            entities=entities,
            parse_mode=parse_mode,
            reply_markup=reply_markup,
            reply_to_message_id=reply_to_message_id,
            text=text,
        )

        return await self._call_api(
            "sendMessage",
            request,
            response_cls=SendMessageResponse,
        )

    async def sendPhoto(
        self,
        *,
        chat_id: int | str,
        photo: str | Path | IO,
        caption: None | str = None,  # 0-1024
        parse_mode: None | str = None,
        caption_entities: None | list[MessageEntity] = None,
        disable_notification: None | bool = None,
        reply_to_message_id: None | int = None,
        allow_sending_without_reply: None | bool = None,
        reply_markup: None | ReplyMarkupType = None,
    ) -> Message:
        """
        Use this method to send photos.

        https://core.telegram.org/bots/api#sendphoto

        :param chat_id: Unique identifier for the target chat
        or username of the target channel
        (in the format @channelusername).

        :param photo: Photo to send.
        Pass a file_id as String to send a photo
        that exists on the Telegram servers (recommended),
        pass an HTTP URL as a String for Telegram
        to get a photo from the Internet,
        or upload a new photo using multipart/form-data.

        The photo must be at most 10 MB in size.
        The photo's width and height must not exceed 10000 in total.
        Width and height ratio must be at most 20.

        :param caption: Photo caption
        (may also be used when resending photos by file_id),
        0-1024 characters after entities parsing.

        :param parse_mode: Mode for parsing entities in the message text.
        See formatting options for more details.

        :param caption_entities: A JSON-serialized list of special entities
        that appear in the caption,
        which can be specified instead of parse_mode.

        :param disable_notification: Sends the message silently.
        Users will receive a notification with no sound.

        :param reply_to_message_id: If the message is a reply,
        ID of the original message.

        :param allow_sending_without_reply: Pass True,
        if the message should be sent
        even if the specified replied-to message is not found.

        :param reply_markup: Additional interface options.
        A JSON-serialized object for an inline keyboard,
        custom reply keyboard,
        instructions to remove reply keyboard
        or to force a reply from the user.

        :return: on success, the Message sent.
        """

        request = SendPhotoRequest(
            allow_sending_without_reply=allow_sending_without_reply,
            caption=caption,
            caption_entities=caption_entities,
            chat_id=chat_id,
            disable_notification=disable_notification,
            parse_mode=parse_mode,
            photo=photo,
            reply_markup=reply_markup,
            reply_to_message_id=reply_to_message_id,
        )

        return await self._call_api(
            "sendPhoto", request, response_cls=SendPhotoResponse
        )

    async def setWebhook(self, *, url: str) -> bool:
        request = SetWebhookRequest(
            url=url,
        )

        return await self._call_api(
            "setWebhook",
            request,
            response_cls=SetWebhookResponse,
        )

    _T = TypeVar(
        "_T"
    )  # don't worry about this: used as a generic type var in `_call_api`

    async def _call_api(
        self,
        method: str,
        request: None | Request = None,
        *,
        response_cls: Type[Response[_T]] = Response[_T],
        timeout: None | int = None,
    ) -> _T:
        """
        Performs the call to the Bot API returning a value of proper type.
        In case of error raises `Bot.RequestError`.

        https://core.telegram.org/bots/api#making-requests

        :param method: name of the supported Telegram Bot API method

        :param request: request object,
        composed of input params of public method

        :param response_cls: desired response class with actual result type

        :return: object of response class' result type
        """

        url = f"{self.api_url}/{method}"

        try:
            # for methods which do not need the request at all
            request = request or Request()

            async with self.client_session() as session:
                data: aiohttp.FormData | bytes

                with request.files() as files:
                    if files:
                        headers = {}
                        data = aiohttp.FormData(
                            {_f: str(_v) for _f, _v in request.dict().items()}
                        )
                        for field, stream in files.items():
                            data.add_field(field, stream, filename="InputFile")
                    else:
                        headers = {"Content-Type": "application/json"}
                        data = request.jsonb()

                    kw = {}
                    if timeout:
                        kw["timeout"] = timeout * 2

                    send_request = session.post(
                        url,
                        data=data,
                        headers=headers,
                        **kw,
                    )

                    async with send_request as http_response:
                        body = await http_response.read()

                        if http_response.status != 200:
                            raise self.RequestError(body.decode())

            payload = orjson.loads(body)
            if not payload:
                err = f"unexpected empty payload on /{method}"
                raise self.RequestError(err)

            # actual&valid Telegram response
            response = response_cls.parse_obj(payload)

            if not response.ok:
                raise self.RequestError(response.description)

            if response.result is None:
                err = f"unexpected null result on /{method} -> {response}"
                raise self.RequestError(err)

            return response.result

        except Exception as err:
            raise self.RequestError(err) from err

    async def _download_file(
        self,
        file_path: str,
    ) -> BytesIO:
        """
        Downloads a file using file path from API.

        https://core.telegram.org/bots/api#getfile
        https://core.telegram.org/bots/api#file

        :param file_path: File path.
        Use https://api.telegram.org/file/bot<token>/<file_path>
        to get the file.

        :return: a BytesIO object with file content.
        """

        try:
            url = f"{self.file_url}/{file_path}"

            async with self.client_session() as session:
                async with session.get(url) as http_response:
                    body = await http_response.read()
                    if http_response.status != 200:
                        raise self.RequestError(body.decode())

            buffer = BytesIO()
            buffer.write(body)
            buffer.seek(0)

            return buffer

        except Exception as err:
            raise self.RequestError(err) from err



from typing import Type

from pydantic import Field

from oyabun.telegram import File
from oyabun.telegram.base import Response
from oyabun.telegram.entities import Chat
from oyabun.telegram.entities import Message
from oyabun.telegram.entities import Update
from oyabun.telegram.entities import User
from oyabun.telegram.entities import WebhookInfo


class AnswerCallbackQueryResponse(Response[bool]):
    result: bool = Field(False)


class DeleteWebhookResponse(Response[bool]):
    result: bool = Field(False)


class EditMessageCaptionResponse(Response[bool | Message]):
    result: None | bool | Message = Field(None)


class EditMessageReplyMarkupResponse(Response[bool | Message]):
    result: None | bool | Message = Field(None)


class EditMessageTextResponse(Response[bool | Message]):
    result: None | bool | Message = Field(None)


class GetChatResponse(Response[Chat]):
    result: None | Chat = Field(None)


class GetFileResponse(Response[File]):
    result: None | File = Field(None)


class GetMeResponse(Response[User]):
    result: None | User = Field(None)


class GetUpdatesResponse(Response[list[Update]]):
    result: list[Update] = Field(default_factory=list)


class GetWebhookInfoResponse(Response[WebhookInfo]):
    result: None | WebhookInfo = Field(None)


class SendMessageResponse(Response[Message]):
    result: None | Message = Field(None)


class SendPhotoResponse(Response[Message]):
    result: None | Message = Field(None)


class SetWebhookResponse(Response[bool]):
    result: bool = Field(False)


__models__: set[Type[Response]] = {
    AnswerCallbackQueryResponse,
    DeleteWebhookResponse,
    EditMessageCaptionResponse,
    EditMessageReplyMarkupResponse,
    EditMessageTextResponse,
    GetChatResponse,
    GetFileResponse,
    GetMeResponse,
    GetUpdatesResponse,
    GetWebhookInfoResponse,
    SendMessageResponse,
    SendPhotoResponse,
    SetWebhookResponse,
}

__all__ = (
    "__models__",
    "AnswerCallbackQueryResponse",
    "DeleteWebhookResponse",
    "EditMessageCaptionResponse",
    "EditMessageReplyMarkupResponse",
    "EditMessageTextResponse",
    "GetChatResponse",
    "GetFileResponse",
    "GetMeResponse",
    "GetUpdatesResponse",
    "GetWebhookInfoResponse",
    "SendMessageResponse",
    "SendPhotoResponse",
    "SetWebhookResponse",
)



from typing import Type

from oyabun.telegram.base import __models__ as __models__base
from oyabun.telegram.base import Request
from oyabun.telegram.base import Response
from oyabun.telegram.base import ResponseParameters
from oyabun.telegram.base import TelegramBotApiType
from oyabun.telegram.entities import __models__ as __models__entities
from oyabun.telegram.entities import Audio
from oyabun.telegram.entities import BotCommand
from oyabun.telegram.entities import BotCommandScope
from oyabun.telegram.entities import BotCommandScopeDefault
from oyabun.telegram.entities import CallbackQuery
from oyabun.telegram.entities import Chat
from oyabun.telegram.entities import ChatLocation
from oyabun.telegram.entities import ChatPermissions
from oyabun.telegram.entities import ChatPhoto
from oyabun.telegram.entities import Dice
from oyabun.telegram.entities import Document
from oyabun.telegram.entities import File
from oyabun.telegram.entities import ForceReply
from oyabun.telegram.entities import InlineKeyboardButton
from oyabun.telegram.entities import InlineKeyboardMarkup
from oyabun.telegram.entities import KeyboardButton
from oyabun.telegram.entities import Location
from oyabun.telegram.entities import MaskPosition
from oyabun.telegram.entities import Message
from oyabun.telegram.entities import MessageEntity
from oyabun.telegram.entities import PhotoSize
from oyabun.telegram.entities import ReplyKeyboardMarkup
from oyabun.telegram.entities import ReplyKeyboardRemove
from oyabun.telegram.entities import Sticker
from oyabun.telegram.entities import Update
from oyabun.telegram.entities import User
from oyabun.telegram.entities import Video
from oyabun.telegram.entities import VideoNote
from oyabun.telegram.entities import Voice
from oyabun.telegram.entities import WebhookInfo
from oyabun.telegram.requests import __models__ as __models__requests
from oyabun.telegram.requests import AnswerCallbackQueryRequest
from oyabun.telegram.requests import DeleteWebhookRequest
from oyabun.telegram.requests import EditMessageCaptionRequest
from oyabun.telegram.requests import EditMessageReplyMarkupRequest
from oyabun.telegram.requests import EditMessageTextRequest
from oyabun.telegram.requests import GetChatRequest
from oyabun.telegram.requests import GetFileRequest
from oyabun.telegram.requests import GetMeRequest
from oyabun.telegram.requests import GetUpdatesRequest
from oyabun.telegram.requests import GetWebhookInfoRequest
from oyabun.telegram.requests import SendMessageRequest
from oyabun.telegram.requests import SendPhotoRequest
from oyabun.telegram.requests import SetWebhookRequest
from oyabun.telegram.responses import __models__ as __models__responses
from oyabun.telegram.responses import AnswerCallbackQueryResponse
from oyabun.telegram.responses import DeleteWebhookResponse
from oyabun.telegram.responses import EditMessageCaptionResponse
from oyabun.telegram.responses import EditMessageReplyMarkupResponse
from oyabun.telegram.responses import EditMessageTextResponse
from oyabun.telegram.responses import GetChatResponse
from oyabun.telegram.responses import GetFileResponse
from oyabun.telegram.responses import GetMeResponse
from oyabun.telegram.responses import GetUpdatesResponse
from oyabun.telegram.responses import GetWebhookInfoResponse
from oyabun.telegram.responses import SendMessageResponse
from oyabun.telegram.responses import SendPhotoResponse
from oyabun.telegram.responses import SetWebhookResponse

__models__: set[Type[TelegramBotApiType]] = (
    __models__base
    | __models__entities  # noqa: W503
    | __models__requests  # noqa: W503
    | __models__responses  # noqa: W503
)

for _model in __models__:
    _model.update_forward_refs()

__all__ = (
    "AnswerCallbackQueryRequest",
    "AnswerCallbackQueryResponse",
    "Audio",
    "BotCommand",
    "BotCommandScope",
    "BotCommandScopeDefault",
    "CallbackQuery",
    "Chat",
    "ChatLocation",
    "ChatPermissions",
    "ChatPhoto",
    "DeleteWebhookRequest",
    "DeleteWebhookResponse",
    "Dice",
    "Document",
    "EditMessageCaptionRequest",
    "EditMessageCaptionResponse",
    "EditMessageReplyMarkupRequest",
    "EditMessageReplyMarkupResponse",
    "EditMessageTextRequest",
    "EditMessageTextResponse",
    "File",
    "ForceReply",
    "GetChatRequest",
    "GetChatResponse",
    "GetFileRequest",
    "GetFileResponse",
    "GetMeRequest",
    "GetMeResponse",
    "GetUpdatesRequest",
    "GetUpdatesResponse",
    "GetWebhookInfoRequest",
    "GetWebhookInfoResponse",
    "InlineKeyboardButton",
    "InlineKeyboardMarkup",
    "KeyboardButton",
    "Location",
    "MaskPosition",
    "Message",
    "MessageEntity",
    "PhotoSize",
    "ReplyKeyboardMarkup",
    "ReplyKeyboardRemove",
    "Request",
    "Response",
    "ResponseParameters",
    "SendMessageRequest",
    "SendMessageResponse",
    "SendPhotoRequest",
    "SendPhotoResponse",
    "SetWebhookRequest",
    "SetWebhookResponse",
    "Sticker",
    "Update",
    "User",
    "Video",
    "VideoNote",
    "Voice",
    "WebhookInfo",
)



from pathlib import Path
from typing import NamedTuple


class VersionTuple(NamedTuple):
    major: int = 2022
    minor: int = 8
    micro: int = 20
    dev: None | int = None

    def __str__(self) -> str:
        ver = f"{self.major}.{self.minor}.{self.micro}"
        if self.dev is not None:
            ver = f"{ver}.dev{self.dev}"
        return ver


version = VersionTuple()

VERSION = str(version)


def verify_version() -> None:
    # TODO: change to tomllib in Python 3.11  # noqa: T101
    import toml

    this_file = Path(__file__)
    repo = this_file.parent.parent.resolve()
    pyproject_toml = repo / "pyproject.toml"
    if "oyabun" not in pyproject_toml.as_posix():
        # ignore for site-packages installations
        return

    assert pyproject_toml.is_file()
    with pyproject_toml.open("r") as stream:
        data = toml.load(stream)

    version_pyproject_toml = (
        data.get("tool", {}).get("poetry", {}).get("version")
    )

    err = (
        f"incorrect versions: "
        f"{version_pyproject_toml!r} (in {pyproject_toml.as_posix()}) "
        f"!= {VERSION!r} (in {this_file.as_posix()})"
    )
    assert version_pyproject_toml == VERSION, err



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['oyabun', 'oyabun.telegram']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp[speedups]>=3.8.1,<4.0.0',
 'orjson>=3.7.11,<4.0.0',
 'pydantic>=1.9.1,<2.0.0']

setup_kwargs = {
    'name': 'oyabun',
    'version': '2022.8.20',
    'description': 'Telegram library with strict API',
    'long_description': "# 親分\n\n_— the extreme path_\n\nA library for building Telegram bots.\n\n![shimizu no jirocho](https://github.com/tgrx/oyabun/raw/main/docs/img/shimizu_no_jirocho.jpg)\n\n![build status](https://github.com/tgrx/oyabun/actions/workflows/development.yaml/badge.svg?branch=main)\n\n## Mission\n\nThe mission of this library is to provide a strict interface for the API.\nBy *strict* we mean that all types and methods in the library interface\nare mapped to those described in the Telegram API docs.\n\nYou won't meet any auxiliary stuff like sophisticated OOP/async patterns,\nobscure event loops and listeners and the kind of stuff like that.\n\nAPI types are Pydantic models with strict type hints.\n\nAPI methods accept params with exactly the same type and name as described in API.\n\nAny optional field/param is marked as `Optional` or `None | T`. Don't be afraid of tri-state bool types.\n",
    'author': 'Alexander Sidorov',
    'author_email': 'alexander@sidorov.dev',
    'maintainer': 'Alexander Sidorov',
    'maintainer_email': 'alexander@sidorov.dev',
    'url': 'https://github.com/tgrx/oyabun',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime
from typing import Optional
from typing import Type

from pydantic import Field

from oyabun.telegram.base import TelegramBotApiType


class Audio(TelegramBotApiType):
    """
    This object represents an audio file
    to be treated as music by the Telegram clients.

    https://core.telegram.org/bots/api#audio
    """

    duration: int = Field(...)
    file_id: str = Field(...)
    file_name: None | str = Field(None)
    file_size: None | int = Field(None)
    file_unique_id: str = Field(...)
    mime_type: None | str = Field(None)
    performer: None | str = Field(None)
    thumb: Optional["PhotoSize"] = Field(None)
    title: None | str = Field(None)


class BotCommand(TelegramBotApiType):
    command: str = Field(..., min_length=1, max_length=32)
    description: str = Field(..., min_length=3, max_length=256)


class BotCommandScope(TelegramBotApiType):
    type: str = Field(...)  # noqa: A003,VNE003


class BotCommandScopeDefault(BotCommandScope):
    type: str = Field("default")  # noqa: A003,VNE003


class CallbackQuery(TelegramBotApiType):
    """
    This object represents an incoming callback query
    from a callback button in an inline keyboard.

    If the button that originated the query
    was attached to a message sent by the bot,
    the field message will be present.

    If the button was attached to a message
    sent via the bot (in inline mode),
    the field inline_message_id will be present.

    Exactly one of the fields data or game_short_name will be present.

    https://core.telegram.org/bots/api#callbackquery
    """

    chat_instance: str = Field(...)
    data: None | str = Field(None)
    from_: "User" = Field(..., alias="from")
    game_short_name: None | str = Field(None)
    id: str = Field(...)  # noqa: A003,VNE003
    inline_message_id: None | str = Field(None)
    message: Optional["Message"] = Field(None)

    class Config:
        fields = {
            "from_": "from",
        }


class Chat(TelegramBotApiType):
    """
    This object represents a chat.

    https://core.telegram.org/bots/api#chat
    """

    bio: None | str = Field(None)
    can_set_sticker_set: None | bool = Field(None)
    description: None | bool = Field(None)
    first_name: None | str = Field(None)
    has_private_forwards: None | bool = Field(None)
    has_protected_content: None | bool = Field(None)
    has_restricted_voice_and_video_messages: None | bool = Field(None)
    id: int = Field(...)  # noqa: A003, VNE003
    invite_link: None | bool = Field(None)
    join_by_request: None | bool = Field(None)
    join_to_send_messages: None | bool = Field(None)
    last_name: None | str = Field(None)
    linked_chat_id: None | int = Field(None)
    location: Optional["ChatLocation"] = Field(None)
    message_auto_delete_time: None | int = Field(None)
    permissions: Optional["ChatPermissions"] = Field(None)
    photo: Optional["ChatPhoto"] = Field(None)
    pinned_message: Optional["Message"] = Field(None)
    slow_mode_delay: None | int = Field(None)
    sticker_set_name: None | str = Field(None)
    title: None | str = Field(None)
    type: str = Field(...)  # noqa: A003, VNE003
    username: None | str = Field(None)


class ChatLocation(TelegramBotApiType):
    """
    Represents a location to which a chat is connected.

    https://core.telegram.org/bots/api#chatlocation
    """

    address: str = Field(...)
    location: "Location" = Field(...)


class ChatPermissions(TelegramBotApiType):
    """
    Describes actions that a non-administrator user is allowed to take in a chat.

    https://core.telegram.org/bots/api#chatpermissions
    """

    can_add_web_page_previews: None | bool = Field(None)
    can_change_info: None | bool = Field(None)
    can_invite_users: None | bool = Field(None)
    can_pin_messages: None | bool = Field(None)
    can_send_media_messages: None | bool = Field(None)
    can_send_messages: None | bool = Field(None)
    can_send_other_messages: None | bool = Field(None)
    can_send_polls: None | bool = Field(None)


class ChatPhoto(TelegramBotApiType):
    """
    This object represents a chat photo.

    https://core.telegram.org/bots/api#chatphoto
    """

    big_file_id: str = Field(...)
    big_file_unique_id: str = Field(...)
    small_file_id: str = Field(...)
    small_file_unique_id: str = Field(...)


class Dice(TelegramBotApiType):
    """
    This object represents an animated emoji that displays a random value.

    https://core.telegram.org/bots/api#dice
    """

    emoji: str = Field(...)
    value: int = Field(...)


class Document(TelegramBotApiType):
    """
    This object represents a general file
    (as opposed to photos, voice messages and audio files).

    https://core.telegram.org/bots/api#document
    """

    file_id: str = Field(...)
    file_name: None | str = Field(None)
    file_size: None | int = Field(None)
    file_unique_id: str = Field(...)
    mime_type: None | str = Field(None)
    thumb: Optional["PhotoSize"] = Field(None)


class File(TelegramBotApiType):
    file_id: str = Field(...)
    file_path: None | str = Field(None)
    file_size: None | int = Field(None)
    file_unique_id: str = Field(...)


class ForceReply(TelegramBotApiType):
    """
    Upon receiving a message with this object,
    Telegram clients will display a reply interface to the user
    (act as if the user has selected the bot's message and tapped 'Reply').

    This can be extremely useful
    if you want to create user-friendly step-by-step interfaces
    without having to sacrifice privacy mode.

    Example: A poll bot for groups runs in privacy mode
    (only receives commands, replies to its messages and mentions).

    There could be two ways to create a new poll:

    -   Explain the user how to send a command with parameters
        (e.g. /newpoll question answer1 answer2).

        May be appealing for hardcore users but lacks modern day polish.

    -   Guide the user through a step-by-step process.
        'Please send me your question',
        'Cool, now let's add the first answer option',
        'Great. Keep adding answer options,
         then send /done when you're ready'.

        The last option is definitely more attractive.

        And if you use ForceReply in your bot's questions,
        it will receive the user's answers
        even if it only receives replies, commands and mentions —
        without any extra work for the user.

    https://core.telegram.org/bots/api#forcereply
    """

    force_reply: bool = Field(True)


class InlineKeyboardButton(TelegramBotApiType):
    """
    This object represents one button of an inline keyboard.

    You MUST use exactly one of the optional fields.

    https://core.telegram.org/bots/api#inlinekeyboardbutton
    """

    callback_data: None | str = Field(None, min_length=1, max_length=64)
    text: str = Field(...)
    url: None | str = Field(None)


class InlineKeyboardMarkup(TelegramBotApiType):
    """
    This object represents an inline keyboard
    that appears right next to the message it belongs to.

    https://core.telegram.org/bots/api#inlinekeyboardmarkup
    """

    inline_keyboard: list[list[InlineKeyboardButton]] = Field(
        default_factory=list,
    )


class KeyboardButton(TelegramBotApiType):
    """
    This object represents one button of the reply keyboard.

    For simple text buttons String can be used
    instead of this object to specify text of the button.

    Optional fields request_contact, request_location,
    and request_poll are mutually exclusive.

    https://core.telegram.org/bots/api#keyboardbutton
    """

    text: str = Field(...)


class Location(TelegramBotApiType):
    """
    This object represents a point on the map.

    https://core.telegram.org/bots/api#location
    """

    heading: None | int = Field(None, ge=1, le=360)
    horizontal_accuracy: None | float = Field(None, ge=0, le=1500)
    latitude: float = Field(...)
    live_period: None | int = Field(None)
    longitude: float = Field(...)
    proximity_alert_radius: None | int = Field(None)


class MaskPosition(TelegramBotApiType):
    """
    This object describes the position on faces where a mask should be placed by default.

    https://core.telegram.org/bots/api#maskposition
    """

    point: str = Field(...)
    scale: float = Field(...)
    x_shift: float = Field(...)
    y_shift: float = Field(...)


class Message(TelegramBotApiType):
    """
    This object represents a message.

    https://core.telegram.org/bots/api#message
    """

    audio: None | Audio = Field(None)
    caption: None | str = Field(None)
    chat: Chat = Field(...)
    date: datetime = Field(...)
    dice: None | Dice = Field(None)
    document: None | Document = Field(None)
    edit_date: None | datetime = Field(None)
    entities: Optional[list["MessageEntity"]] = Field(None)
    from_: Optional["User"] = Field(None, alias="from")
    message_id: int = Field(...)
    photo: Optional[list["PhotoSize"]] = Field(None)
    reply_markup: Optional["InlineKeyboardMarkup"] = Field(None)
    reply_to_message: Optional["Message"] = Field(None)
    sticker: Optional["Sticker"] = Field(None)
    text: None | str = Field(None)
    video: Optional["Video"] = Field(None)
    video_note: Optional["VideoNote"] = Field(None)
    voice: Optional["Voice"] = Field(None)

    class Config:
        fields = {
            "from_": "from",
        }


class MessageEntity(TelegramBotApiType):
    """
    This object represents one special entity in a text message.

    For example, hashtags, usernames, URLs, etc.

    https://core.telegram.org/bots/api#messageentity
    """

    language: None | str = Field(None)
    length: int = Field(...)
    offset: int = Field(...)
    type: str = Field(...)  # noqa: A003, VNE003
    url: None | str = Field(None)
    user: Optional["User"] = Field(None)


class PhotoSize(TelegramBotApiType):
    """
    This object represents one size of a photo or a file / sticker thumbnail.

    https://core.telegram.org/bots/api#photosize
    """

    file_id: str = Field(...)
    file_size: None | int = Field(None)
    file_unique_id: str = Field(...)
    height: int = Field(...)
    width: int = Field(...)


class ReplyKeyboardMarkup(TelegramBotApiType):
    """
    This object represents a custom keyboard with reply options
    (see Introduction to bots for details and examples).

    https://core.telegram.org/bots#keyboards
    https://core.telegram.org/bots/api#replykeyboardmarkup
    """

    keyboard: list[list[KeyboardButton]] = Field(default_factory=list)
    one_time_keyboard: bool = Field(False)
    resize_keyboard: bool = Field(False)


class ReplyKeyboardRemove(TelegramBotApiType):
    """
    Upon receiving a message with this object,
    Telegram clients will remove the current custom keyboard
    and display the default letter-keyboard.

    By default, custom keyboards are displayed
    until a new keyboard is sent by a bot.

    An exception is made for one-time keyboards
    that are hidden immediately
    after the user presses a button (see ReplyKeyboardMarkup).

    https://core.telegram.org/bots/api#replykeyboardremove
    """

    remove_keyboard: bool = Field(True)


class Sticker(TelegramBotApiType):
    """
    This object represents a sticker.

    https://core.telegram.org/bots/api#sticker
    """

    custom_emoji_id: None | str = Field(None)
    emoji: None | str = Field(None)
    file_id: str = Field(...)
    file_size: None | int = Field(None)
    file_unique_id: str = Field(...)
    height: int = Field(...)
    is_animated: bool = Field(...)
    is_premium: None | bool = Field(None)
    is_video: bool = Field(...)
    mask_position: None | MaskPosition = Field(None)
    premium_animation: None | File = Field(None)
    set_name: None | str = Field(None)
    thumb: None | PhotoSize = Field(None)
    type: str = Field(...)  # noqa: A003,VNE003
    width: int = Field(...)


class Update(TelegramBotApiType):
    """
    This object represents an incoming update.

    At most one of the optional parameters
    can be present in any given update.

    https://core.telegram.org/bots/api#update
    """

    callback_query: None | CallbackQuery = Field(None)
    channel_post: None | Message = Field(None)
    edited_channel_post: None | Message = Field(None)
    edited_message: None | Message = Field(None)
    message: None | Message = Field(None)
    update_id: int = Field(...)

    def get_chat(self) -> Chat:
        if self.message:
            return self.message.chat

        if self.edited_message:
            return self.edited_message.chat

        if (cbq := self.callback_query) and cbq.message:
            return cbq.message.chat

        raise ValueError(f"cannot get chat from {self}")

    def get_user(self) -> "User":
        if (obj := self.message) and obj.from_:
            return obj.from_

        if (obj := self.edited_message) and obj.from_:
            return obj.from_

        if self.callback_query:
            return self.callback_query.from_

        raise ValueError(f"cannot get user from {self}")


class User(TelegramBotApiType):
    """
    This object represents a Telegram user or bot.

    https://core.telegram.org/bots/api#user
    """

    added_to_attachment_menu: None | bool = Field(None)
    can_join_groups: None | bool = Field(None)
    can_read_all_group_messages: None | bool = Field(None)
    first_name: str = Field(...)
    id: int = Field(...)  # noqa: A003, VNE003
    is_bot: bool = Field(...)
    is_premium: None | bool = Field(None)
    language_code: None | str = Field(None)
    last_name: None | str = Field(None)
    supports_inline_queries: None | bool = Field(None)
    username: None | str = Field(None)


class Video(TelegramBotApiType):
    """
    This object represents a video file.

    https://core.telegram.org/bots/api#video
    """

    duration: int = Field(...)
    file_id: str = Field(...)
    file_name: None | str = Field(None)
    file_size: None | int = Field(None)
    file_unique_id: str = Field(...)
    height: int = Field(...)
    mime_type: None | str = Field(None)
    thumb: None | PhotoSize = Field(None)
    width: int = Field(...)


class VideoNote(TelegramBotApiType):
    """
    This object represents a video message.

    https://core.telegram.org/bots/api#videonote
    """

    duration: int = Field(...)
    file_id: str = Field(...)
    file_size: None | int = Field(None)
    file_unique_id: str = Field(...)
    length: int = Field(...)
    thumb: None | PhotoSize = Field(None)


class Voice(TelegramBotApiType):
    """
    This object represents a voice note.

    https://core.telegram.org/bots/api#voice
    """

    duration: int = Field(...)
    file_id: str = Field(...)
    file_size: None | int = Field(None)
    file_unique_id: str = Field(...)
    mime_type: None | str = Field(None)


class WebhookInfo(TelegramBotApiType):
    """
    Contains information about the current status of a webhook.

    https://core.telegram.org/bots/api#webhookinfo
    """

    allowed_updates: list[str] | None = Field(None)
    has_custom_certificate: bool = Field(...)
    ip_address: None | str = Field(None)
    last_error_date: None | datetime = Field(None)
    last_error_message: None | str = Field(None)
    max_connections: None | int = Field(None)
    pending_update_count: int = Field(0)
    url: str = Field(...)


ReplyMarkupType = (
    ForceReply
    | InlineKeyboardMarkup  # noqa: W503
    | ReplyKeyboardMarkup  # noqa: W503
    | ReplyKeyboardRemove  # noqa: W503
)

__models__: set[Type[TelegramBotApiType]] = {
    Audio,
    BotCommand,
    BotCommandScope,
    BotCommandScopeDefault,
    CallbackQuery,
    Chat,
    ChatLocation,
    ChatPermissions,
    ChatPhoto,
    Dice,
    Document,
    File,
    ForceReply,
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    KeyboardButton,
    Location,
    MaskPosition,
    Message,
    MessageEntity,
    PhotoSize,
    ReplyKeyboardMarkup,
    ReplyKeyboardRemove,
    Sticker,
    Update,
    User,
    Video,
    VideoNote,
    Voice,
    WebhookInfo,
}

__all__ = (
    "__models__",
    "Audio",
    "BotCommand",
    "BotCommandScope",
    "BotCommandScopeDefault",
    "CallbackQuery",
    "Chat",
    "ChatLocation",
    "ChatPermissions",
    "ChatPhoto",
    "Dice",
    "Document",
    "File",
    "ForceReply",
    "InlineKeyboardButton",
    "InlineKeyboardMarkup",
    "KeyboardButton",
    "Location",
    "MaskPosition",
    "Message",
    "MessageEntity",
    "PhotoSize",
    "ReplyKeyboardMarkup",
    "ReplyKeyboardRemove",
    "ReplyMarkupType",
    "Sticker",
    "Update",
    "User",
    "Video",
    "VideoNote",
    "Voice",
    "WebhookInfo",
)
